import java.io.*;

/**
 * Functions to convert data from little endian byte ordering.
 * NOTE: The only functions that have been tested so far are
 * readInt() and readUnsignedShort().  The others still need to 
 * be verified.
 */
public class LEDataInputStream implements DataInput {
    InputStream input;
    DataInputStream dataIn; 

    LEDataInputStream(InputStream is) { 
        input = is;
        dataIn = new DataInputStream(is);
    }

    public boolean readBoolean() throws IOException { 
        return dataIn.readBoolean(); 
    }

    public byte readByte() throws IOException { return (byte)input.read(); }
    public int readUnsignedByte() throws IOException  { return input.read() & 0xff; }

    public int readUnsignedShort() throws IOException {
        byte[] bytes = new byte[2];
        if (input.read(bytes) < 2) throw new IOException("Couldn't read bytes");
        return (bytes[0] & 0xff) | ((bytes[1] & 0xff) << 8);

    }
    public int readInt() throws IOException {
        byte[] bytes = new byte[4];
        if (input.read(bytes) < 4) throw new IOException("Couldn't read bytes");
        return ((bytes[3] & 0xff) << 24 ) | ((bytes[2] & 0xff) << 16) | 
            ((bytes[1] & 0xff) << 8) | (bytes[0] & 0xff);
    }
    public long readLong() throws IOException {
        byte[] bytes = new byte[8];
        if (input.read(bytes) < 8) throw new IOException("Couldn't read bytes");
        return ((bytes[7] & 0xff) << 56 ) | ((bytes[6] & 0xff) << 48) | 
            ((bytes[5] & 0xff) << 40) | ((bytes[4] & 0xff) << 32) |
            ((bytes[3] & 0xff) << 24) | ((bytes[2] & 0xff) << 16) |
            ((bytes[1] & 0xff) << 8) | (bytes[0] & 0xff);
    }

    public char readChar() throws IOException { 
        return (char)readUnsignedShort(); 
    }

    public double readDouble() throws IOException { 
        return Double.longBitsToDouble(readLong());
    }
    public float readFloat() throws IOException {
        return Float.intBitsToFloat(readInt());
    }

    public void readFully(byte  b[]) throws IOException  { dataIn.readFully(b); }
    public void readFully(byte  b[], int  off, int  len) throws IOException  {
        dataIn.readFully(b, off, len);
    }


    public String readLine() throws IOException { return dataIn.readLine(); }

    public short readShort() throws IOException {  
        return (short)readUnsignedShort(); 
    }


    public String readUTF() throws IOException {
        return null; // not implemented
    }

    public int skipBytes(int  n) throws IOException { 
        return dataIn.skipBytes(n); 
    }

}
