import java.awt.*;
import java.io.*;
import java.net.*;
import java.awt.image.*;
       
public class RLTest extends Frame implements Runnable {
    RLImage rli;
    Image img = null;
    int xOffset = 10;
    int yOffset = 30;

    RLTest(String imagefile) {
        super("RLImage Example");

        try {
            rli = new RLImage(new BufferedInputStream(new FileInputStream(imagefile)), true);
            System.out.println(rli.toString());
            img = createImage(rli);
            resize(rli.width + xOffset*2, rli.height + yOffset*3/2);
            show();

//            new Thread(this).start();   // Start changing colors
        } catch (Exception e) {
            e.printStackTrace();
        }

    }


    public void paint(Graphics g) {
        try {
            if (img == null)
                img = getToolkit().getImage(new URL(
                "http://csgrad.cs.vt.edu/~jnielsen/images/fampict.jpeg"
//                  "http://simon.cs.vt.edu/~jnielsen/images/pumpkin.gif"
                    ));
            g.drawImage(img, xOffset, yOffset, this);
            // Still need to figure out why this won't draw incrementally
            // We know that imageUpdate is getting called for the component
            // as each set of pixels arrives.  
            // And according to the documentation for Component.imageUpdate():
            //
            //   The imageUpdate method of Component incrementally draws 
            //   an image on the component as more of the bits of the image 
            //   are available. 
            //   
            //   If the system property "awt.image.incrementalDraw" is 
            //   missing or has the value "true", the image is incrementally 
            //   drawn.  
            //
            // This happens with GIFs and JPEGs but not with RLImages!!
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Overrides Component.update so that it doesn't clear the
     * background before calling paint().  Makes for 
     * less flickering.
     */ 
    public void update(Graphics g) { paint(g); }

    /**
     * Watch the image being delivered
     */
    public boolean imageUpdate(Image img, int infoflags, 
        int x, int y, int width, int height) {
        System.out.print("x="+x+" ");
        System.out.print("y="+y+" ");
        System.out.print("width="+width+" ");
        System.out.print("height="+height+" ");
        System.out.print("  infoflags=");
        if ((infoflags & ABORT ) != 0) System.out.print("ABORT ");
        if ((infoflags & ALLBITS ) != 0) System.out.print("ALLBITS ");
        if ((infoflags & ERROR ) != 0) System.out.print("ERROR ");
        if ((infoflags & FRAMEBITS ) != 0) System.out.print("FRAMEBITS ");
        if ((infoflags & HEIGHT ) != 0) System.out.print("HEIGHT ");
        if ((infoflags & PROPERTIES ) != 0) System.out.print("PROPERTIES ");
        if ((infoflags & SOMEBITS ) != 0) System.out.print("SOMEBITS ");
        if ((infoflags & WIDTH ) != 0) System.out.print("WIDTH ");
        System.out.println();
        return super.imageUpdate(img, infoflags, x, y, width, height);
    }

    public boolean handleEvent(Event evt) {
        if (evt.id == Event.WINDOW_DESTROY) {
                dispose();
                System.exit(0);
                return true;
            }   
        else return super.handleEvent(evt);
    }

    public boolean mouseMove(Event evt, int x, int y) {
        x -= xOffset;
        y -= yOffset;
        System.out.print("Mouse at " + x + ", " + y +
                ": ");
        try {
            int val = rli.getPixel(x, y);
            System.out.print("value is " + val + ", ");
        } catch (Exception e) {
            System.out.print("outside of image");
        }
        System.out.println();

        return true;
    }



    public static void main(String[] args) {
        if (args.length < 1) {
            System.out.println("USAGE: Java RLTest <imagefile>");
            return;
        }
        new RLTest(args[0]);
    }

    public void run() {
        // Change the image's colors
        while (true) try {
//            Thread.sleep(1000);
            rli.changeColorModel(new DirectColorModel(16, 0xf, 0xf00, 0xf0));
//            Thread.sleep(1000);
            rli.changeColorModel(new DirectColorModel(16, 0xf0, 0xf, 0xf00));
//            Thread.sleep(1000);
            rli.changeColorModel(new DirectColorModel(16, 0xf00, 0xf0, 0xf));
        } catch (Exception e) {
        }
    }
            

}


